package edu.utexas.cs.surdules.pipes.demo;

import edu.utexas.cs.surdules.pipes.model.Connector;
import edu.utexas.cs.surdules.pipes.model.Distributions;
import edu.utexas.cs.surdules.pipes.model.InvalidModelException;
import edu.utexas.cs.surdules.pipes.model.Model;
import edu.utexas.cs.surdules.pipes.model.Widget;
import edu.utexas.cs.surdules.pipes.model.simulate.Simulator;
import edu.utexas.cs.surdules.pipes.model.statistics.StatletFactory;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetAllocate;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetDelay;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetRelease;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetResource;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetSink;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetSource;

import java.util.Iterator;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class CarWashProblem {
    private static void connect(Widget a, Widget b) throws InvalidModelException {
        Connector x = null, y = null;

        for (Iterator iter = a.getConnectors().iterator(); iter.hasNext();) {
            Connector c = (Connector) iter.next();
            if (c.getDirection() == Connector.OUTGOING) {
                x = c;
                break;
            }
        }

        for (Iterator iter = b.getConnectors().iterator(); iter.hasNext();) {
            Connector c = (Connector) iter.next();
            if (c.getDirection() == Connector.INCOMING) {
                y = c;
                break;
            }
        }

        if (x == null || y == null) {
            throw new InvalidModelException("Widgets do not have valid connectors!");
        }

        x.connect(y);
    }

    public static void main(String[] args) {
        try {
            Model model = new Model();

            WidgetSource cars = new WidgetSource(model);
            cars.setName("Cars");
            cars.setDistributionName(Distributions.NEGATIVEEXPONENTIAL);
            cars.setDistributionValueA(11);
            cars.setNumberOfTransactions(2000);
            model.addWidget(cars);

            WidgetAllocate allocate = new WidgetAllocate(model);
            allocate.setName("Allocate");
            allocate.setDistributionName(Distributions.FIXED);
            allocate.setDistributionValueA(1);
            allocate.getStatistics().enableStatlet(StatletFactory.QUEUEPOPULATION);
            allocate.getStatistics().enableStatlet(StatletFactory.RESPONSETIME);
            model.addWidget(allocate);

            WidgetDelay wash = new WidgetDelay(model);
            wash.setName("Wash");
            wash.setDistributionName(Distributions.FIXED);
            wash.setDistributionValueA(10);
            model.addWidget(wash);

            WidgetRelease release = new WidgetRelease(model);
            release.setName("Release");
            release.setDistributionName(Distributions.FIXED);
            release.setDistributionValueA(1);
            model.addWidget(release);

            WidgetSink sink = new WidgetSink(model);
            sink.setName("Sink");
            model.addWidget(sink);

            WidgetResource washers = new WidgetResource(model);
            washers.setName("Washers");
            washers.setNumberOfTokens(1);
            model.addWidget(washers);

            allocate.setResource(washers);
            release.setResource(washers);

            connect(cars, allocate);
            connect(allocate, wash);
            connect(wash, release);
            connect(release, sink);

            Simulator simulator = new Simulator(model);

            simulator.start();
            while (!simulator.isFinished()) {
                simulator.step();
            }
            simulator.end();

            System.out.println(allocate.getStatistics());
        } catch (Throwable t) {
            System.err.println("An unexpected exception occured:");
            t.printStackTrace();
            System.exit(1);
        }
    }
}
