package edu.utexas.cs.surdules.pipes.model;

import edu.utexas.cs.surdules.pipes.model.forms.PropertiesItem;
import edu.utexas.cs.surdules.pipes.model.forms.PropertiesList;
import edu.utexas.cs.surdules.pipes.model.io.Visitor;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetSource;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class Arc {
    private static final double EPSILON = 0.05;
    private static final double PAD = 2.0;

    private Connector m_start;
    private Connector m_end;

    private Model m_model;

    private double m_selectProbability = 1.0;
    private String m_selectCategory;

    public Arc(Connector start, Connector end) {
        m_start = start;
        m_end = end;

        m_model = m_start.getWidget().getModel();
    }

    public Connector getStart() {
        return m_start;
    }

    public Connector getEnd() {
        return m_end;
    }

    public boolean contains(Point point) {
        Point _start = m_start.getCenter();
        int startX = m_start.getWidget().getX() + m_start.getX() + (int) _start.getX();
        int startY = m_start.getWidget().getY() + m_start.getY() + (int) _start.getY();

        Point _end = m_end.getCenter();
        int endX = m_end.getWidget().getX() + m_end.getX() + (int) _end.getX();
        int endY = m_end.getWidget().getY() + m_end.getY() + (int) _end.getY();

        double directionX = endX - startX;
        double directionY = endY - startY;
        double directionLength = Math.sqrt(directionX * directionX + directionY * directionY);

        double deltaX = point.getX() - startX;
        double deltaY = point.getY() - startY;
        double deltaLength = Math.sqrt(deltaX * deltaX + deltaY * deltaY);

        double dot = deltaX * directionX + deltaY * directionY;

        double projectionLength = dot / directionLength;
        if (projectionLength < 0 || projectionLength > directionLength) {
            return false;
        }

        double distance = Math.sqrt(deltaLength * deltaLength - projectionLength * projectionLength);
        if (distance <= 2) {
            return true;
        } else {
            return false;
        }
    }

    public PropertiesList getPropertiesList() {
        PropertiesList propertiesList = new PropertiesList();

        propertiesList.addItem(new PropertiesItem("Select Probability", Double.class, new Double(getSelectProbability())));

        List categories = new ArrayList();
        categories.add(PropertiesItem.ANY);
        for (Iterator iter = m_model.getWidgets(WidgetSource.class).iterator(); iter.hasNext();) {
            WidgetSource source = (WidgetSource) iter.next();
            String category = source.getTransactionCategory();
            if (category != null) {
                categories.add(category);
            }
        }

        String category = getSelectCategory();
        if (category != null && !categories.contains(category)) {
            categories.add(category);
        }

        propertiesList.addItem(new PropertiesItem("Select Category", String.class,
                (getSelectCategory() == null? PropertiesItem.ANY : getSelectCategory()), categories));

        return propertiesList;
    }

    public void setPropertiesList(PropertiesList propertiesList) {
        setSelectProbability(((Double) propertiesList.getItem("Select Probability").getValue()).doubleValue());

        String category = (String) propertiesList.getItem("Select Category").getValue();
        if (category.equals(PropertiesItem.ANY)) {
            setSelectCategory(null);
        } else {
            setSelectCategory(category);
        }
    }

    public double getSelectProbability() {
        return m_selectProbability;
    }

    public void setSelectProbability(double selectProbability) {
        if (selectProbability < 0.0 || selectProbability > 1.0) {
            throw new IllegalArgumentException("Select probability must be between 0 and 1: " + selectProbability);
        }
        m_selectProbability = selectProbability;
    }

    public String getSelectCategory() {
        return m_selectCategory;
    }

    public void setSelectCategory(String category) {
        m_selectCategory = category;
    }

    public void accept(Visitor visitor) {
        visitor.visitArc(this);
    }
}
