package edu.utexas.cs.surdules.pipes.model.forms;

import java.util.Iterator;
import java.util.List;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class PropertiesItem {
    public static final String NONE = "[None]";
    public static final String ANY = "[Any]";
    public static final String CREATENEW = "[Create New]";

    private String m_name;
    private Class m_valueType;
    private Object m_value;

    private List m_values;

    public PropertiesItem(String name, Class valueType, Object value) {
        if (name == null) {
            throw new IllegalArgumentException("Name must not be null");
        }

        if (valueType == null) {
            throw new IllegalArgumentException("ValueType must not be null");
        }

        m_name = name;
        m_valueType = valueType;

        if (!checkValue(value)) {
            throw new IllegalArgumentException("Invalid Value object: " + value);
        }

        m_value = value;
    }

    public PropertiesItem(String name, Class valueType, Object value, List values) {
        this(name, valueType, value);

        if (values == null || values.size() == 0) {
            throw new IllegalArgumentException("Values must not be null or empty");
        }

        m_values = values;
    }

    public String getName() {
        return m_name;
    }

    public Class getValueType() {
        return m_valueType;
    }

    public Object getValue() {
        return m_value;
    }

    public void setValue(Object value) {
        if (!checkValue(value)) {
            throw new IllegalArgumentException("Invalid or mismatched value: " + value);
        }

        m_value = coerce(value);
    }

    private Object coerce(Object value) {
        if (value.getClass().equals(m_valueType)) {
            return value;
        }

        try {
            if (m_valueType.equals(String.class)) {
                return value.toString();
            } else if (m_valueType.equals(Byte.class)) {
                return Byte.valueOf(value.toString());
            } else if (m_valueType.equals(Double.class)) {
                return Double.valueOf(value.toString());
            } else if (m_valueType.equals(Float.class)) {
                return Float.valueOf(value.toString());
            } else if (m_valueType.equals(Integer.class)) {
                return Integer.valueOf(value.toString());
            } else if (m_valueType.equals(Long.class)) {
                return Long.valueOf(value.toString());
            } else if (m_valueType.equals(Short.class)) {
                return Short.valueOf(value.toString());
            } else {
                return null;
            }
        } catch (NumberFormatException nfe) {
            return null;
        }
    }

    public boolean checkValue(Object value) {
        if (value == null) {
            return false;
        }

        value = coerce(value);
        if (value == null) {
            return false;
        }

        if (isMultiValued()) {
            for (Iterator iter = m_values.iterator(); iter.hasNext();) {
                if (iter.next().equals(value)) {
                    return true;
                }
            }

            return false;
        } else {
            return true;
        }
    }

    public boolean isMultiValued() {
        return (m_values != null);
    }

    public List getValues() {
        return m_values;
    }
}
