package edu.utexas.cs.surdules.pipes.model.io;

import edu.utexas.cs.surdules.pipes.model.Model;
import edu.utexas.cs.surdules.pipes.model.Widget;
import edu.utexas.cs.surdules.pipes.model.WidgetFactory;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetAllocate;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetDelay;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetRelease;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetResource;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetService;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetSink;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetSource;
import org.w3c.dom.Document;

import java.io.File;
import java.io.IOException;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public final class XMLFactory {
    private XMLFactory() {
        // nop
    }

    public static Document createDocument() {
        return new electric.xml.Document();
    }

    public static Document loadDocument(File file) throws IOException {
        try {
            return new electric.xml.Document(file);
        } catch (electric.xml.ParseException pe) {
            throw new IOException(pe.getMessage());
        }
    }

    public static void saveDocument(Document document, File file) throws IOException {
        ((electric.xml.Document) document).write(file);
    }

    /*
    public static Document createDocument() {
        try {
            javax.xml.parsers.DocumentBuilderFactory factory =
                    javax.xml.parsers.DocumentBuilderFactory.newInstance();
            javax.xml.parsers.DocumentBuilder builder = factory.newDocumentBuilder();
            return builder.newDocument();
        } catch (javax.xml.parsers.FactoryConfigurationError fce) {
            throw new RuntimeException(fce.getMessage());
        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            throw new RuntimeException(pce.getMessage());
        }
    }

    public static Document loadDocument(File file) throws IOException, SAXException {
        try {
            javax.xml.parsers.DocumentBuilderFactory factory =
                    javax.xml.parsers.DocumentBuilderFactory.newInstance();
            javax.xml.parsers.DocumentBuilder builder = factory.newDocumentBuilder();
            return builder.parse(file);
        } catch (javax.xml.parsers.FactoryConfigurationError fce) {
            throw new RuntimeException(fce.getMessage());
        } catch (javax.xml.parsers.ParserConfigurationException pce) {
            throw new RuntimeException(pce.getMessage());
        }
    }

    public static void saveDocument(Document document, File file) throws IOException {
        org.apache.xml.serialize.OutputFormat format = new org.apache.xml.serialize.OutputFormat();
        format.setLineSeparator(System.getProperty("line.separator"));
        format.setIndenting(true);
        format.setLineWidth(0);

        FileWriter writer = new FileWriter(file);
        try {
            org.apache.xml.serialize.XMLSerializer serializer = new org.apache.xml.serialize.XMLSerializer(
                    writer, format);
            serializer.serialize(document);
        } finally {
            writer.close();
        }
    }
    */

    public static String createXMLTypeForWidget(Widget widget) {
        if (widget instanceof WidgetService) {
            return "service";
        } else if (widget instanceof WidgetDelay) {
            return "delay";
        } else if (widget instanceof WidgetAllocate) {
            return "allocate";
        } else if (widget instanceof WidgetRelease) {
            return "release";
        } else if (widget instanceof WidgetSource) {
            return "source";
        } else if (widget instanceof WidgetSink) {
            return "sink";
        } else if (widget instanceof WidgetResource) {
            return "resource";
        } else {
            throw new IllegalArgumentException("Invalid Widget type: " + widget.getClass().getName());
        }
    }

    public static Widget createWidgetForXMLType(String type, Model model) {
        if (type.equals("service")) {
            return WidgetFactory.createWidgetService(model);
        } else if (type.equals("delay")) {
            return WidgetFactory.createWidgetDelay(model);
        } else if (type.equals("allocate")) {
            return WidgetFactory.createWidgetAllocate(model);
        } else if (type.equals("release")) {
            return WidgetFactory.createWidgetRelease(model);
        } else if (type.equals("source")) {
            return WidgetFactory.createWidgetSource(model);
        } else if (type.equals("sink")) {
            return WidgetFactory.createWidgetSink(model);
        } else if (type.equals("resource")) {
            return WidgetFactory.createWidgetResource(model);
        } else {
            throw new IllegalArgumentException("Invalid Widget type: '" + type + "'");
        }
    }
}
