package edu.utexas.cs.surdules.pipes.model.simulate;

import edu.utexas.cs.surdules.pipes.model.Widget;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class Event {
    public static final int ENTER = 0;
    public static final int DEQUEUE = 1;
    public static final int LEAVE = 2;

    private Simulator m_context;
    private Widget m_widget;
    private int m_transactionID;
    private String m_transactionCategory;
    private int m_transactionPriority;
    private double m_time;
    private int m_state;

    public Event(Simulator context, Widget widget, int id, String category, int priority) {
        m_context = context;
        m_widget = widget;
        m_transactionID = id;
        m_transactionCategory = category;
        m_transactionPriority = priority;
        m_time = 0;
        m_state = ENTER;
    }

    public Event(Event event, double time, int state) {
        m_context = event.getContext();
        m_widget = event.getWidget();
        m_transactionID = event.getTransactionID();
        m_transactionCategory = event.getTransactionCategory();
        m_time = time;
        if (state == ENTER || state == DEQUEUE || state == LEAVE) {
            m_state = state;
        } else {
            throw new IllegalArgumentException("Invalid state: " + state);
        }
    }

    public Event(Event event, Widget widget, int state) {
        m_context = event.getContext();
        m_widget = widget;
        m_transactionID = event.getTransactionID();
        m_transactionCategory = event.getTransactionCategory();
        m_time = event.getTime();
        if (state == ENTER || state == DEQUEUE || state == LEAVE) {
            m_state = state;
        } else {
            throw new IllegalArgumentException("Invalid state: " + state);
        }
    }

    public Widget getWidget() {
        return m_widget;
    }

    public int getTransactionID() {
        return m_transactionID;
    }

    public String getTransactionCategory() {
        return m_transactionCategory;
    }

    public int getTransactionPriority() {
        return m_transactionPriority;
    }

    public double getTime() {
        return m_time;
    }

    public Simulator getContext() {
        return m_context;
    }

    public int getState() {
        return m_state;
    }

    public Event advance() {
        return m_widget.advance(this, m_context);
    }

    private String stateToString() {
        switch (m_state) {
            case ENTER:
                return "E";
            case DEQUEUE:
                return "D";
            case LEAVE:
                return "L";
            default:
                // this should never happen
                throw new Error("Unrecognized state: " + m_state);
        }
    }

    public String toString() {
        return "Event: [widget:" + m_widget.getName() +
                "][transaction id:" + m_transactionID +
                "][transaction category:" + m_transactionCategory +
                "][transaction priority:" + m_transactionPriority +
                "][time:" + m_time +
                "][state:" + stateToString() + "]";
    }
}
