package edu.utexas.cs.surdules.pipes.model.statistics;

import edu.utexas.cs.surdules.pipes.model.simulate.Event;

import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class QueuePopulationStatlet implements Statlet {
    private Set m_pairs = new TreeSet(new TimePopulationPairComparator());
    private int m_population;
    private double m_end;

    public String getName() {
        return "Queue Population Count";
    }

    public void onStart() {
        m_pairs.clear();
        m_population = 0;
        m_end = 0;
    }

    public void onCreate(Event event) {
        // nop
    }

    public void onArrive(Event event) {
        // nop
    }

    public void onLeave(Event event) {
        // nop
    }

    public void onEnqueue(Event event) {
        m_population++;

        m_pairs.add(new TimePopulationPair(event.getTime(), m_population));
    }

    public void onDequeue(Event event) {
        m_population--;
        if (m_population < 0) {
            throw new IllegalStateException("No events previously enqueued: " + event);
        }

        m_pairs.add(new TimePopulationPair(event.getTime(), m_population));
    }

    public void onWork(Event event, double length) {
        // nop
    }

    public void onDestroy(Event event) {
        // nop
    }

    public void onEnd(double time) {
        m_end = time;
    }

    public double getResult() {
        TimePopulationPair previous = null, current = null;

        double sum = 0;
        for (Iterator iter = m_pairs.iterator(); iter.hasNext();) {
            previous = current;
            current = (TimePopulationPair) iter.next();

            if (previous == null) {
                continue;
            }

            sum = sum + (previous.population * (current.time - previous.time));
        }

        return (m_end == 0 ? Double.NaN : sum / m_end);
    }

    public String toString() {
        double result = getResult();
        return "QueuePopulationStatlet: sum=" + (m_end * result) + ",end=" + m_end + ",average=" + result;
    }
}
