package edu.utexas.cs.surdules.pipes.model.statistics;

import edu.utexas.cs.surdules.pipes.model.simulate.Event;

import java.util.HashMap;
import java.util.Map;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class ResponseTimeStatlet implements Statlet {
    private int m_count;
    private double m_think;
    private Map m_intervals = new HashMap();

    public String getName() {
        return "Response Time";
    }

    public void onStart() {
        m_count = 0;
        m_think = 0;
        m_intervals.clear();
    }

    public void onCreate(Event event) {
        // nop
    }

    public void onArrive(Event event) {
        Integer key = new Integer(event.getTransactionID());
        if (m_intervals.containsKey(key)) {
            throw new IllegalArgumentException("Event already seen: " + event);
        }

        m_intervals.put(key, new Double(event.getTime()));
    }

    public void onLeave(Event event) {
        Double arrive = (Double) m_intervals.remove(new Integer(event.getTransactionID()));
        if (arrive == null) {
            throw new IllegalStateException("Event not seen before: " + event);
        }

        m_think = m_think + (event.getTime() - arrive.doubleValue());
        m_count++;
    }

    public void onEnqueue(Event event) {
        // nop
    }

    public void onDequeue(Event event) {
        // nop
    }

    public void onWork(Event event, double length) {
        // nop
    }

    public void onDestroy(Event event) {
        // nop
    }

    public void onEnd(double time) {
        // nop
    }

    public double getResult() {
        return (m_count == 0 ? Double.NaN : (m_think / m_count));
    }

    public String toString() {
        return "ResponseTimeStatlet: count=" + m_count + ",think=" + m_think + ",average=" + getResult();
    }
}
