package edu.utexas.cs.surdules.pipes.model.statistics;

import edu.utexas.cs.surdules.pipes.model.Widget;
import edu.utexas.cs.surdules.pipes.model.simulate.Event;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class Statistics {
    private Widget m_widget;
    private Map m_statlets = new HashMap();

    public Statistics(Widget widget) {
        if (widget == null) {
            throw new IllegalArgumentException("Widget cannot be null");
        }

        m_widget = widget;
    }

    public Statistics(Statistics copy) {
        m_widget = copy.m_widget;

        for (Iterator iter = copy.m_statlets.keySet().iterator(); iter.hasNext();) {
            String name = (String) iter.next();
            enableStatlet(name);
        }
    }

    public boolean isEnabled(String name) {
        return m_statlets.containsKey(name);
    }

    public void enableStatlet(String name) {
        if (m_statlets.containsKey(name)) {
            throw new IllegalArgumentException("Statistic '" + name + "' already enabled");
        }

        m_statlets.put(name, StatletFactory.createStatlet(name));
    }

    public void disableStatistic(String name) {
        if (!m_statlets.containsKey(name)) {
            throw new IllegalArgumentException("Statistic '" + name + "' not enabled");
        }

        m_statlets.remove(name);
    }

    public void onStart() {
        for (Iterator iter = m_statlets.values().iterator(); iter.hasNext();) {
            Statlet statlet = (Statlet) iter.next();
            statlet.onStart();
        }
    }

    public void onCreate(Event event) {
        if (event.getTime() > m_widget.getModel().getWarmupTime()) {
            for (Iterator iter = m_statlets.values().iterator(); iter.hasNext();) {
                Statlet statlet = (Statlet) iter.next();
                statlet.onCreate(event);
            }
        }
    }

    public void onArrive(Event event) {
        if (event.getTime() > m_widget.getModel().getWarmupTime()) {
            for (Iterator iter = m_statlets.values().iterator(); iter.hasNext();) {
                Statlet statlet = (Statlet) iter.next();
                statlet.onArrive(event);
            }
        }
    }

    public void onLeave(Event event) {
        if (event.getTime() > m_widget.getModel().getWarmupTime()) {
            for (Iterator iter = m_statlets.values().iterator(); iter.hasNext();) {
                Statlet statlet = (Statlet) iter.next();
                statlet.onLeave(event);
            }
        }
    }

    public void onEnqueue(Event event) {
        if (event.getTime() > m_widget.getModel().getWarmupTime()) {
            for (Iterator iter = m_statlets.values().iterator(); iter.hasNext();) {
                Statlet statlet = (Statlet) iter.next();
                statlet.onEnqueue(event);
            }
        }
    }

    public void onDequeue(Event event) {
        if (event.getTime() > m_widget.getModel().getWarmupTime()) {
            for (Iterator iter = m_statlets.values().iterator(); iter.hasNext();) {
                Statlet statlet = (Statlet) iter.next();
                statlet.onDequeue(event);
            }
        }
    }

    public void onWork(Event event, double length) {
        if (event.getTime() > m_widget.getModel().getWarmupTime()) {
            for (Iterator iter = m_statlets.values().iterator(); iter.hasNext();) {
                Statlet statlet = (Statlet) iter.next();
                statlet.onWork(event, length);
            }
        }
    }

    public void onDestroy(Event event) {
        if (event.getTime() > m_widget.getModel().getWarmupTime()) {
            for (Iterator iter = m_statlets.values().iterator(); iter.hasNext();) {
                Statlet statlet = (Statlet) iter.next();
                statlet.onDestroy(event);
            }
        }
    }

    public void onEnd(double time) {
        for (Iterator iter = m_statlets.values().iterator(); iter.hasNext();) {
            Statlet statlet = (Statlet) iter.next();
            statlet.onEnd(time);
        }
    }

    public int getStatletCount() {
        return m_statlets.size();
    }

    public Map getStatlets() {
        return m_statlets;
    }

    public String toString() {
        StringBuffer result = new StringBuffer();
        if (m_statlets.size() > 0) {
            result.append("Statistics for widget '" + m_widget.getName() + "':" + System.getProperty("line.separator"));
            for (Iterator iter = m_statlets.values().iterator(); iter.hasNext();) {
                Statlet statlet = (Statlet) iter.next();
                result.append(statlet.toString()).append(System.getProperty("line.separator"));
            }
        }
        return result.toString();
    }
}
