package edu.utexas.cs.surdules.pipes.model.widgets;

import edu.utexas.cs.surdules.pipes.model.Arc;
import edu.utexas.cs.surdules.pipes.model.Connector;
import edu.utexas.cs.surdules.pipes.model.Distributions;
import edu.utexas.cs.surdules.pipes.model.Model;
import edu.utexas.cs.surdules.pipes.model.Widget;
import edu.utexas.cs.surdules.pipes.model.forms.PropertiesItem;
import edu.utexas.cs.surdules.pipes.model.forms.PropertiesList;
import edu.utexas.cs.surdules.pipes.model.io.Visitor;
import edu.utexas.cs.surdules.pipes.model.simulate.Simulator;
import edu.utexas.cs.surdules.pipes.model.simulate.Event;

import java.util.Arrays;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class WidgetDelay extends Widget {
    private String m_distributionName;
    private double m_distributionValueA, m_distributionValueB;

    public WidgetDelay(Model model) {
        this(model, 0, 0);
    }

    public WidgetDelay(Model model, int x, int y) {
        super(model, x, y);

        m_distributionName = Distributions.getDistributionNames()[0];
    }

    protected void createConnectors() {
        Connector incoming = new Connector(this, Connector.INCOMING, 10, Connector.WEST, 0);
        m_connectors.add(incoming);

        Connector outgoing = new Connector(this, Connector.OUTGOING, 10, Connector.EAST, 1);
        m_connectors.add(outgoing);
    }

    protected String getDefaultName() {
        return "Delay";
    }

    public PropertiesList getPropertiesList() {
        PropertiesList propertiesList = super.getPropertiesList();
        propertiesList.addItem(new PropertiesItem("Delay Time Distribution", String.class, m_distributionName,
                Arrays.asList(Distributions.getDistributionNames())));
        propertiesList.addItem(new PropertiesItem("Distribution Parameter a", Double.class, new Double(m_distributionValueA)));
        propertiesList.addItem(new PropertiesItem("Distribution Parameter b", Double.class, new Double(m_distributionValueB)));
        return propertiesList;
    }

    public void setPropertiesList(PropertiesList propertiesList) {
        super.setPropertiesList(propertiesList);

        String distributionName = (String) propertiesList.getItem("Delay Time Distribution").getValue();
        double distributionValueA = ((Double) propertiesList.getItem("Distribution Parameter a").getValue()).doubleValue();
        double distributionValueB = ((Double) propertiesList.getItem("Distribution Parameter b").getValue()).doubleValue();

        Distributions distributions = new Distributions();
        // This will throw an IllegalArgumentException if the distribution
        // parameters are incorrect.
        distributions.computeNamedDistribution(distributionName, distributionValueA, distributionValueB);

        m_distributionName = distributionName;
        m_distributionValueA = distributionValueA;
        m_distributionValueB = distributionValueB;
    }

    public String getDistributionName() {
        return m_distributionName;
    }

    public void setDistributionName(String distributionName) {
        m_distributionName = distributionName;
    }

    public double getDistributionValueA() {
        return m_distributionValueA;
    }

    public void setDistributionValueA(double distributionValueA) {
        m_distributionValueA = distributionValueA;
    }

    public double getDistributionValueB() {
        return m_distributionValueB;
    }

    public void setDistributionValueB(double distributionValueB) {
        m_distributionValueB = distributionValueB;
    }

    public void accept(Visitor visitor) {
        visitor.visitWidgetDelay(this);
        super.accept(visitor);
    }

    public Event advance(Event event, Simulator context) {
        if (event.getWidget() != this) {
            throw new IllegalArgumentException("Cannot process Event belonging to another Widget");
        }

        if (event.getState() == Event.ENTER) {
            m_statistics.onArrive(event);

            double time = event.getTime() +
                    context.getDistributions().computeNamedDistribution(
                            getDistributionName(),
                            getDistributionValueA(),
                            getDistributionValueB());
            return new Event(event, time, Event.LEAVE);
        } else if (event.getState() == Event.DEQUEUE) {
            throw new IllegalArgumentException("Cannot handle dequeued event on a widget without a queue: " + event);
        } else if (event.getState() == Event.LEAVE) {
            Arc arc = selectOutgoingArc(event.getTransactionCategory(), context);

            if (arc == null) {
                m_statistics.onDestroy(event);
                return null;
            } else {
                m_statistics.onLeave(event);
                return new Event(event, arc.getEnd().getWidget(), Event.ENTER);
            }
        } else {
            throw new IllegalArgumentException("Unrecognized event state: " + event.getState());
        }
    }
}
