package edu.utexas.cs.surdules.pipes.model.widgets;

import edu.utexas.cs.surdules.pipes.model.Model;
import edu.utexas.cs.surdules.pipes.model.Widget;
import edu.utexas.cs.surdules.pipes.model.forms.PropertiesItem;
import edu.utexas.cs.surdules.pipes.model.forms.PropertiesList;
import edu.utexas.cs.surdules.pipes.model.io.Visitor;
import edu.utexas.cs.surdules.pipes.model.simulate.Simulator;
import edu.utexas.cs.surdules.pipes.model.simulate.Event;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class WidgetResource extends Widget {
    private int m_numberOfTokens;

    public WidgetResource(Model model) {
        this(model, 0, 0);
    }

    public WidgetResource(Model model, int x, int y) {
        super(model, x, y);
    }

    protected void createConnectors() {
        // nop
    }

    protected String getDefaultName() {
        return "Resource";
    }

    public PropertiesList getPropertiesList() {
        PropertiesList propertiesList = super.getPropertiesList();
        propertiesList.addItem(new PropertiesItem("Number of Tokens", Integer.class, new Integer(m_numberOfTokens)));
        return propertiesList;
    }

    public void setPropertiesList(PropertiesList propertiesList) {
        super.setPropertiesList(propertiesList);

        setNumberOfTokens(((Integer) propertiesList.getItem("Number of Tokens").getValue()).intValue());
    }

    public int getNumberOfTokens() {
        return m_numberOfTokens;
    }

    public void setNumberOfTokens(int numberOfTokens) {
        if (numberOfTokens < 0) {
            throw new IllegalArgumentException("Number of tokens must be at least 0: " + numberOfTokens);
        }
        m_numberOfTokens = numberOfTokens;
    }

    public void accept(Visitor visitor) {
        visitor.visitWidgetResource(this);
        super.accept(visitor);
    }

    public Event advance(Event event, Simulator context) {
        throw new IllegalStateException("Cannot handle events: " + event);
    }
}
