package edu.utexas.cs.surdules.pipes.model.widgets;

import edu.utexas.cs.surdules.pipes.model.Connector;
import edu.utexas.cs.surdules.pipes.model.Model;
import edu.utexas.cs.surdules.pipes.model.Widget;
import edu.utexas.cs.surdules.pipes.model.io.Visitor;
import edu.utexas.cs.surdules.pipes.model.simulate.Simulator;
import edu.utexas.cs.surdules.pipes.model.simulate.Event;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class WidgetSink extends Widget {
    public WidgetSink(Model model) {
        this(model, 0, 0);
    }

    public WidgetSink(Model model, int x, int y) {
        super(model, x, y);
    }

    protected void createConnectors() {
        Connector incoming = new Connector(this, Connector.INCOMING, 10, Connector.WEST, 0);
        m_connectors.add(incoming);
    }

    protected String getDefaultName() {
        return "Sink";
    }

    public void accept(Visitor visitor) {
        visitor.visitWidgetSink(this);
        super.accept(visitor);
    }

    public Event advance(Event event, Simulator context) {
        if (event.getWidget() != this) {
            throw new IllegalArgumentException("Cannot process Event belonging to another Widget");
        }

        if (event.getState() == Event.ENTER) {
            m_statistics.onArrive(event);
            context.destroyEvent(event);
            m_statistics.onDestroy(event);
            return null;
        } else if (event.getState() == Event.DEQUEUE) {
            throw new IllegalArgumentException("Cannot handle dequeued event on a widget without a queue: " + event);
        } else if (event.getState() == Event.LEAVE) {
            throw new IllegalArgumentException("Cannot sink event: " + event);
        } else {
            throw new IllegalArgumentException("Unrecognized event state: " + event.getState());
        }
    }
}
