package edu.utexas.cs.surdules.pipes.view;

import edu.utexas.cs.surdules.pipes.model.Model;
import edu.utexas.cs.surdules.pipes.model.io.LoadVisitor;
import edu.utexas.cs.surdules.pipes.model.io.SaveVisitor;
import edu.utexas.cs.surdules.pipes.model.simulate.Simulator;
import edu.utexas.cs.surdules.pipes.view.simulate.SimulatorUI;
import org.xml.sax.SAXException;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.filechooser.FileFilter;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyVetoException;
import java.io.File;
import java.io.IOException;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class ApplicationUI extends JFrame {
    private JMenuBar m_menuBar;
    private JToolBar m_toolBar;
    private JDesktopPane m_desktopPane;

    private Model m_currentModel;

    private String m_currentDirectory = System.getProperty("user.dir");

    public ApplicationUI(String title) {
        super(title);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                onExit();
            }
        });

        JMenuBar menuBar = createMenuBar();
        setJMenuBar(menuBar);

        JPanel contentPane = new JPanel();
        contentPane.setLayout(new BorderLayout());

        JToolBar toolBar = createToolBar();
        contentPane.add(toolBar, BorderLayout.NORTH);

        JDesktopPane desktopPane = new JDesktopPane();
        contentPane.add(desktopPane, BorderLayout.CENTER);

        setContentPane(contentPane);

        m_menuBar = menuBar;
        m_toolBar = toolBar;
        m_desktopPane = desktopPane;

        touch();
    }

    private JMenuBar createMenuBar() {
        JMenuBar menuBar = new JMenuBar();

        JMenu menuFile = new JMenu("File");
        menuFile.setMnemonic(KeyEvent.VK_F);
        menuBar.add(menuFile);

        JMenuItem menuFileNew = new JMenuItem("New Model");
        menuFileNew.setMnemonic(KeyEvent.VK_N);
        menuFileNew.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onCreateModel();
            }
        });
        menuFile.add(menuFileNew);

        JMenuItem menuFileLoad = new JMenuItem("Load Model");
        menuFileLoad.setMnemonic(KeyEvent.VK_L);
        menuFileLoad.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    onLoadModel();
                } catch (IOException ioe) {
                    JOptionPane.showMessageDialog(ApplicationUI.this, ioe.getMessage(), "IO Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                } catch (SAXException saxe) {
                    JOptionPane.showMessageDialog(ApplicationUI.this, saxe.getMessage(), "Parse Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });
        menuFile.add(menuFileLoad);

        menuFile.addSeparator();

        JMenuItem menuFileSave = new JMenuItem("Save Model");
        menuFileSave.setMnemonic(KeyEvent.VK_S);
        menuFileSave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    onSaveModel();
                } catch (IOException ioe) {
                    JOptionPane.showMessageDialog(ApplicationUI.this, ioe.getMessage(), "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });
        menuFile.add(menuFileSave);

        JMenuItem menuFileSaveAs = new JMenuItem("Save Model As");
        menuFileSaveAs.setMnemonic(KeyEvent.VK_S);
        menuFileSaveAs.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    onSaveModelAs();
                } catch (IOException ioe) {
                    JOptionPane.showMessageDialog(ApplicationUI.this, ioe.getMessage(), "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });
        menuFile.add(menuFileSaveAs);

        menuFile.addSeparator();

        JMenuItem menuFileExit = new JMenuItem("Exit");
        menuFileExit.setMnemonic(KeyEvent.VK_X);
        menuFileExit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onExit();
            }
        });
        menuFile.add(menuFileExit);

        JMenu menuSimulate = new JMenu("Simulate");
        menuSimulate.setMnemonic(KeyEvent.VK_S);
        menuBar.add(menuSimulate);

        JMenuItem menuSimulateStart = new JMenuItem("Start");
        menuSimulateStart.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onSimulate();
            }
        });
        menuSimulate.add(menuSimulateStart);

        JMenu menuHelp = new JMenu("Help");
        menuHelp.setMnemonic(KeyEvent.VK_H);
        menuBar.add(menuHelp);

        JMenuItem menuHelpAbout = new JMenuItem("About");
        menuHelpAbout.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onAbout();
            }
        });
        menuHelp.add(menuHelpAbout);

        return menuBar;
    }

    private JToolBar createToolBar() {
        JToolBar toolBar = new JToolBar();

        JButton buttonNewModel = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/New24.gif")));
        buttonNewModel.setName("New Model");
        buttonNewModel.setToolTipText("New Model");
        buttonNewModel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onCreateModel();
            }
        });
        toolBar.add(buttonNewModel);

        JButton buttonLoadModel = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/Open24.gif")));
        buttonLoadModel.setName("Load Model");
        buttonLoadModel.setToolTipText("Load Model");
        buttonLoadModel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    onLoadModel();
                } catch (IOException ioe) {
                    JOptionPane.showMessageDialog(ApplicationUI.this, ioe.getMessage(), "IO Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                } catch (SAXException saxe) {
                    JOptionPane.showMessageDialog(ApplicationUI.this, saxe.getMessage(), "Parse Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });
        toolBar.add(buttonLoadModel);

        toolBar.addSeparator();

        JButton buttonSaveModel = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/Save24.gif")));
        buttonSaveModel.setName("Save Model");
        buttonSaveModel.setToolTipText("Save Model");
        buttonSaveModel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    onSaveModel();
                } catch (IOException ioe) {
                    JOptionPane.showMessageDialog(ApplicationUI.this, ioe.getMessage(), "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });
        toolBar.add(buttonSaveModel);

        JButton buttonSaveModelAs = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/SaveAs24.gif")));
        buttonSaveModelAs.setName("Save Model As");
        buttonSaveModelAs.setToolTipText("Save Model As");
        buttonSaveModelAs.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    onSaveModelAs();
                } catch (IOException ioe) {
                    JOptionPane.showMessageDialog(ApplicationUI.this, ioe.getMessage(), "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });
        toolBar.add(buttonSaveModelAs);

        toolBar.addSeparator();

        JButton buttonSimulate = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/Play24.gif")));
        buttonSimulate.setName("Simulate");
        buttonSimulate.setToolTipText("Simulate");
        buttonSimulate.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onSimulate();
            }
        });
        toolBar.add(buttonSimulate);

        toolBar.addSeparator();

        JButton buttonAbout = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/About24.gif")));
        buttonAbout.setName("About");
        buttonAbout.setToolTipText("About");
        buttonAbout.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onAbout();
            }
        });
        toolBar.add(buttonAbout);

        return toolBar;
    }

    private void onExit() {
        System.exit(0);
    }

    private void onAbout() {
        AboutUI ui = new AboutUI(this, "About");
        ui.setSize(300, 300);
        ui.setVisible(true);
    }

    private void onCreateModel() {
        useModel(new Model(), null);
    }

    private void onLoadModel() throws IOException, SAXException {
        String fileName = chooseFile(false);
        if (fileName == null) {
            return;
        }

        Model model = new Model();

        LoadVisitor loadVisitor = new LoadVisitor(fileName);
        loadVisitor.load();

        try {
            model.accept(loadVisitor);
        } catch (RuntimeException re) {
            throw new IOException(re.getMessage());
        }

        useModel(model, fileName);
    }

    private void useModel(Model model, String fileName) {
        ModelUI modelUI = new ModelUI(model, this);
        modelUI.setModelFileName(fileName);

        model.setModelUI(modelUI);

        modelUI.touch();

        modelUI.setSize(640, 480);
        modelUI.setVisible(true);
        m_desktopPane.add(modelUI);

        try {
            modelUI.setSelected(true);
        } catch (PropertyVetoException pve) {
            JOptionPane.showMessageDialog(ApplicationUI.this, pve.getMessage(), "Model Selection Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
            m_desktopPane.remove(modelUI);
        }

        m_currentModel = model;
    }

    private void onSaveModel() throws IOException {
        String fileName = m_currentModel.getModelUI().getModelFileName();
        if (fileName == null) {
            fileName = chooseFile(true);
            if (fileName == null) {
                return;
            }
        }

        saveModel(fileName);
    }

    private void onSaveModelAs() throws IOException {
        String fileName = chooseFile(true);
        if (fileName == null) {
            return;
        }

        saveModel(fileName);
    }

    private void saveModel(String fileName) throws IOException {
        m_currentModel.getModelUI().setModelFileName(fileName);
        SaveVisitor saveVisitor = new SaveVisitor(fileName);

        try {
            m_currentModel.accept(saveVisitor);
        } catch (RuntimeException re) {
            throw new IOException(re.getMessage());
        }

        saveVisitor.save();
    }

    private String chooseFile(boolean save) throws IOException {
        JFileChooser fc = new JFileChooser(m_currentDirectory);

        fc.setFileFilter(new FileFilter() {
            public String getDescription() {
                return "XML files (*.xml)";
            }

            public boolean accept(File f) {
                return f.isDirectory() || f.getName().toLowerCase().endsWith(".xml");
            }
        });

        int returnVal;
        if (save) {
            returnVal = fc.showSaveDialog(this);
        } else {
            returnVal = fc.showOpenDialog(this);
        }

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File selectedFile = fc.getSelectedFile();
            m_currentDirectory = selectedFile.getParentFile().getCanonicalPath();
            return selectedFile.getCanonicalPath();
        } else {
            return null;
        }
    }

    private void onSimulate() {
        Simulator simulator = new Simulator(m_currentModel);
        SimulatorUI simulatorUI = new SimulatorUI(this, "Simulate", simulator);
        simulator.setSimulatorUI(simulatorUI);

        simulatorUI.setSize(600, 300);
        simulatorUI.setVisible(true);
    }

    public void touch() {
        ModelUI currentModelUI = (ModelUI) m_desktopPane.getSelectedFrame();

        if (currentModelUI == null) {
            m_currentModel = null;
        } else {
            m_currentModel = currentModelUI.getModel();
        }

        for (int menuIndex = 0; menuIndex < m_menuBar.getMenuCount(); menuIndex++) {
            JMenu menu = m_menuBar.getMenu(menuIndex);
            if (menu.getText().equals("File")) {
                for (int menuItemIndex = 0; menuItemIndex < menu.getMenuComponentCount(); menuItemIndex++) {
                    if (menu.getMenuComponent(menuItemIndex) instanceof JMenuItem) {
                        JMenuItem menuItem = (JMenuItem) menu.getMenuComponent(menuItemIndex);
                        if (menuItem.getText().startsWith("Save")) {
                            if (m_currentModel == null) {
                                menuItem.setEnabled(false);
                            } else {
                                menuItem.setEnabled(true);
                            }
                        }
                    }
                }
            } else if (menu.getText().equals("Simulate")) {
                for (int menuItemIndex = 0; menuItemIndex < menu.getMenuComponentCount(); menuItemIndex++) {
                    if (menu.getMenuComponent(menuItemIndex) instanceof JMenuItem) {
                        JMenuItem menuItem = (JMenuItem) menu.getMenuComponent(menuItemIndex);
                        if (m_currentModel == null) {
                            menuItem.setEnabled(false);
                        } else {
                            menuItem.setEnabled(true);
                        }
                    }
                }
            }
        }

        for (int index = 0; index < m_toolBar.getComponentCount(); index++) {
            if (m_toolBar.getComponentAtIndex(index) instanceof JButton) {
                JButton button = (JButton) m_toolBar.getComponentAtIndex(index);
                if (button.getName().startsWith("Save") ||
                        button.getName().equals("Simulate")) {
                    if (m_currentModel == null) {
                        button.setEnabled(false);
                    } else {
                        button.setEnabled(true);
                    }
                }
            }
        }
    }
}
