package edu.utexas.cs.surdules.pipes.view;

import edu.utexas.cs.surdules.pipes.model.Connector;
import edu.utexas.cs.surdules.pipes.model.InvalidModelException;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class ConnectorUI extends JPanel {
    private Connector m_connector;

    private boolean m_active;

    public ConnectorUI(Connector connector) {
        super();

        setOpaque(false);

        m_connector = connector;

        setBounds(m_connector.getX(), m_connector.getY(), m_connector.getSize(), m_connector.getSize());

        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                m_connector.startArc();
            }

            public void mouseReleased(MouseEvent e) {
                try {
                    m_connector.finishArc();
                } catch (InvalidModelException ime) {
                    JOptionPane.showMessageDialog(ConnectorUI.this, ime.getMessage(), "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }

            public void mouseEntered(MouseEvent e) {
                m_active = true;
                m_connector.endArc(m_connector);
                repaint();
            }

            public void mouseExited(MouseEvent e) {
                m_active = false;
                m_connector.endArc(null);
                repaint();
            }
        });

        addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent e) {
                m_connector.drawArc(e.getPoint());
            }
        });
    }

    public void paint(Graphics g) {
        super.paint(g);
        if (m_active) {
            g.setColor(new Color(255, 0, 0));
            g.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
        }
    }

    public Point getCenter() {
        return m_connector.getCenter();
    }

    public void touch() {
        if (getX() != m_connector.getX() || getY() != m_connector.getY() ||
                getWidth() != m_connector.getSize() || getHeight() != m_connector.getSize()) {
            setBounds(m_connector.getX(), m_connector.getY(), m_connector.getSize(), m_connector.getSize());

            m_connector.getWidget().getWidgetUI().repaint();
        }
    }
}
