package edu.utexas.cs.surdules.pipes.view;

import edu.utexas.cs.surdules.pipes.model.Arc;
import edu.utexas.cs.surdules.pipes.model.Connector;
import edu.utexas.cs.surdules.pipes.model.InvalidModelException;
import edu.utexas.cs.surdules.pipes.model.Model;
import edu.utexas.cs.surdules.pipes.model.Widget;
import edu.utexas.cs.surdules.pipes.model.WidgetFactory;
import edu.utexas.cs.surdules.pipes.model.forms.PropertiesList;
import edu.utexas.cs.surdules.pipes.view.forms.PropertiesForm;

import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashSet;
import java.util.Iterator;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class ModelUI extends JInternalFrame {
    private ApplicationUI m_parent;
    private Model m_model;
    private String m_modelFileName;

    private JPopupMenu m_modelPopupMenu;
    private JPopupMenu m_arcPopupMenu;

    private int m_clickX, m_clickY;

    public ModelUI(Model model, ApplicationUI parent) {
        super(model.getName(),
                true, // resizable
                true, // closable
                true, // maximizable
                true); // iconifiable

        m_parent = parent;
        m_model = model;

        Container contentPane = getContentPane();
        contentPane.setLayout(null); // absolute positioning

        m_modelPopupMenu = createModelPopupMenu();
        m_arcPopupMenu = createArcPopupMenu();

        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

        addInternalFrameListener(new InternalFrameAdapter() {
            public void internalFrameActivated(InternalFrameEvent e) {
                m_parent.touch();
            }

            public void internalFrameDeactivated(InternalFrameEvent e) {
                m_parent.touch();
            }
        });

        contentPane.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                m_clickX = e.getX();
                m_clickY = e.getY();

                maybeShowPopup(e);
            }

            public void mouseReleased(MouseEvent e) {
                maybeShowPopup(e);
            }

            private void maybeShowPopup(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    if (m_model.selectArc(e.getPoint())) {
                        m_arcPopupMenu.show(e.getComponent(),
                                e.getX(), e.getY());
                    } else {
                        m_modelPopupMenu.show(e.getComponent(),
                                e.getX(), e.getY());
                    }
                }
            }
        });
    }

    private JPopupMenu createModelPopupMenu() {
        JPopupMenu popupMenu = new JPopupMenu();

        JMenuItem popupNewServiceWidget = new JMenuItem("New Service Widget");
        popupNewServiceWidget.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Widget widget = WidgetFactory.createWidgetService(m_model);
                widget.setCoordinates(m_clickX, m_clickY);
                try {
                    m_model.addWidget(widget);
                } catch (InvalidModelException ime) {
                    JOptionPane.showMessageDialog(ModelUI.this,
                            "Corrupted model: " + ime.getMessage(), "Error: Corrupted Model",
                            JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });

        JMenuItem popupNewDelayWidget = new JMenuItem("New Delay Widget");
        popupNewDelayWidget.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Widget widget = WidgetFactory.createWidgetDelay(m_model);
                widget.setCoordinates(m_clickX, m_clickY);
                try {
                    m_model.addWidget(widget);
                } catch (InvalidModelException ime) {
                    JOptionPane.showMessageDialog(ModelUI.this,
                            "Corrupted model: " + ime.getMessage(), "Error: Corrupted Model",
                            JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });

        JMenuItem popupNewAllocateWidget = new JMenuItem("New Allocate Widget");
        popupNewAllocateWidget.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Widget widget = WidgetFactory.createWidgetAllocate(m_model);
                widget.setCoordinates(m_clickX, m_clickY);
                try {
                    m_model.addWidget(widget);
                } catch (InvalidModelException ime) {
                    JOptionPane.showMessageDialog(ModelUI.this,
                            "Corrupted model: " + ime.getMessage(), "Error: Corrupted Model",
                            JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });

        JMenuItem popupNewReleaseWidget = new JMenuItem("New Release Widget");
        popupNewReleaseWidget.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Widget widget = WidgetFactory.createWidgetRelease(m_model);
                widget.setCoordinates(m_clickX, m_clickY);
                try {
                    m_model.addWidget(widget);
                } catch (InvalidModelException ime) {
                    JOptionPane.showMessageDialog(ModelUI.this,
                            "Corrupted model: " + ime.getMessage(), "Error: Corrupted Model",
                            JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });

        JMenuItem popupNewResourceWidget = new JMenuItem("New Resource Widget");
        popupNewResourceWidget.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Widget widget = WidgetFactory.createWidgetResource(m_model);
                widget.setCoordinates(m_clickX, m_clickY);
                try {
                    m_model.addWidget(widget);
                } catch (InvalidModelException ime) {
                    JOptionPane.showMessageDialog(ModelUI.this,
                            "Corrupted model: " + ime.getMessage(), "Error: Corrupted Model",
                            JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });

        JMenuItem popupNewSourceWidget = new JMenuItem("New Source Widget");
        popupNewSourceWidget.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Widget widget = WidgetFactory.createWidgetSource(m_model);
                widget.setCoordinates(m_clickX, m_clickY);
                try {
                    m_model.addWidget(widget);
                } catch (InvalidModelException ime) {
                    JOptionPane.showMessageDialog(ModelUI.this,
                            "Corrupted model: " + ime.getMessage(), "Error: Corrupted Model",
                            JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });

        JMenuItem popupNewSinkWidget = new JMenuItem("New Sink Widget");
        popupNewSinkWidget.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Widget widget = WidgetFactory.createWidgetSink(m_model);
                widget.setCoordinates(m_clickX, m_clickY);
                try {
                    m_model.addWidget(widget);
                } catch (InvalidModelException ime) {
                    JOptionPane.showMessageDialog(ModelUI.this,
                            "Corrupted model: " + ime.getMessage(), "Error: Corrupted Model",
                            JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });

        popupMenu.add(popupNewSourceWidget);
        popupMenu.add(popupNewSinkWidget);
        popupMenu.add(popupNewServiceWidget);
        popupMenu.add(popupNewDelayWidget);
        popupMenu.add(popupNewResourceWidget);
        popupMenu.add(popupNewAllocateWidget);
        popupMenu.add(popupNewReleaseWidget);

        popupMenu.addSeparator();

        JMenuItem popupProperties = new JMenuItem("Properties");
        popupProperties.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                PropertiesList propertiesList = m_model.getPropertiesList();
                PropertiesForm form = new PropertiesForm(m_parent, "Model Properties", propertiesList);

                form.setSize(400, 400);
                form.setVisible(true);

                if (form.isAccepted()) {
                    try {
                        m_model.setPropertiesList(propertiesList);
                    } catch (IllegalArgumentException iae) {
                        JOptionPane.showMessageDialog(ModelUI.this, iae.getMessage(), "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        });
        popupMenu.add(popupProperties);

        return popupMenu;
    }

    private JPopupMenu createArcPopupMenu() {
        JPopupMenu popupMenu = new JPopupMenu();

        JMenuItem popupDeleteArc = new JMenuItem("Delete");
        popupDeleteArc.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    m_model.deleteSelectedArc();
                } catch (InvalidModelException ime) {
                    JOptionPane.showMessageDialog(ModelUI.this, ime.getMessage(), "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                }
            }
        });
        popupMenu.add(popupDeleteArc);

        popupMenu.addSeparator();

        JMenuItem popupProperties = new JMenuItem("Properties");
        popupProperties.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Arc selectedArc = m_model.getSelectedArc();
                if (selectedArc == null) {
                    JOptionPane.showMessageDialog(ModelUI.this, "No arc selected", "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                    return;
                }

                PropertiesList propertiesList = selectedArc.getPropertiesList();
                PropertiesForm form = new PropertiesForm(m_parent, "Arc Properties", propertiesList);

                form.setSize(400, 400);
                form.setVisible(true);

                if (form.isAccepted()) {
                    try {
                        selectedArc.setPropertiesList(propertiesList);
                    } catch (IllegalArgumentException iae) {
                        JOptionPane.showMessageDialog(ModelUI.this, iae.getMessage(), "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        });
        popupMenu.add(popupProperties);

        return popupMenu;
    }

    public JFrame getFrame() {
        return m_parent;
    }

    public void touch() {
        HashSet widgetUIs = new HashSet();
        Component[] components = getContentPane().getComponents();
        for (int i = 0; i < components.length; i++) {
            if (components[i] instanceof WidgetUI) {
                widgetUIs.add(components[i]);
            }
        }

        boolean revalidate = false;
        boolean repaint = false;

        for (Iterator widgets = m_model.getWidgets().iterator(); widgets.hasNext();) {
            Widget widget = (Widget) widgets.next();

            if (widget.getWidgetUI() == null) {
                WidgetUI widgetUI = WidgetUIFactory.createWidgetUIFromWidget(widget);
                widget.setWidgetUI(widgetUI);
                getContentPane().add(widgetUI);
                revalidate = true;
            } else if (widgetUIs.contains(widget.getWidgetUI())) {
                widgetUIs.remove(widget.getWidgetUI());
                revalidate = true;
            }
        }

        for (Iterator widgets = widgetUIs.iterator(); widgets.hasNext();) {
            getContentPane().remove((WidgetUI) widgets.next());
            revalidate = true;
        }

        if (!m_model.getName().equals(getTitle())) {
            setTitle(m_model.getName());
            repaint = true;
        }

        if (revalidate) {
            revalidate();
            repaint();
        } else if (repaint) {
            repaint();
        }
    }

    public void paint(Graphics g) {
        super.paint(g);

        for (Iterator widgets = m_model.getWidgets().iterator(); widgets.hasNext();) {
            Widget widget = (Widget) widgets.next();

            for (Iterator connectors = widget.getConnectors().iterator(); connectors.hasNext();) {
                Connector connector = (Connector) connectors.next();
                if (connector.getDirection() == Connector.INCOMING) {
                    continue;
                }

                ConnectorUI startUI = connector.getConnectorUI();
                Point start = SwingUtilities.convertPoint(startUI, startUI.getCenter(), this);

                for (Iterator arcs = connector.getArcs().iterator(); arcs.hasNext();) {
                    Arc arc = (Arc) arcs.next();

                    ConnectorUI endUI = arc.getEnd().getConnectorUI();
                    Point end = SwingUtilities.convertPoint(endUI, endUI.getCenter(), this);

                    g.drawLine((int) start.getX(), (int) start.getY(), (int) end.getX(), (int) end.getY());
                }
            }
        }

        if (m_model.getStartArc() != null && m_model.getDrawArc() != null) {
            ConnectorUI connectorUI = m_model.getStartArc().getConnectorUI();

            Point start = SwingUtilities.convertPoint(connectorUI, connectorUI.getCenter(), this);
            Point end = SwingUtilities.convertPoint(connectorUI, m_model.getDrawArc(), this);

            g.drawLine((int) start.getX(), (int) start.getY(), (int) end.getX(), (int) end.getY());
        }
    }

    public Model getModel() {
        return m_model;
    }

    public String getModelFileName() {
        return m_modelFileName;
    }

    public void setModelFileName(String fileName) {
        m_modelFileName = fileName;
    }
}
