package edu.utexas.cs.surdules.pipes.view;

import edu.utexas.cs.surdules.pipes.model.Connector;
import edu.utexas.cs.surdules.pipes.model.Widget;
import edu.utexas.cs.surdules.pipes.model.forms.PropertiesList;
import edu.utexas.cs.surdules.pipes.model.statistics.Statistics;
import edu.utexas.cs.surdules.pipes.view.forms.PropertiesForm;
import edu.utexas.cs.surdules.pipes.view.forms.StatisticsForm;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.MediaTracker;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.Iterator;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public abstract class WidgetUI extends JPanel {
    protected Widget m_widget;

    private JLabel m_image;
    private JLabel m_name;

    private JPopupMenu m_popupMenu;

    private int m_clickX, m_clickY;

    public WidgetUI(Widget widget) {
        super(new BorderLayout());
        setOpaque(false);

        m_widget = widget;

        ImageIcon imageIcon = new ImageIconFlip(this.getClass().getResource(getIconResourceName()));
        while (imageIcon.getImageLoadStatus() != MediaTracker.COMPLETE) {
            try {
                Thread.sleep(1);
            } catch (InterruptedException ie) {
                // nop
            }
        }

        m_image = new JLabel(imageIcon);
        m_image.setOpaque(false);
        m_image.setLayout(null); // absolute positioning
        m_image.setSize(imageIcon.getIconWidth(), imageIcon.getIconHeight());

        m_widget.setDimensions(imageIcon.getIconWidth(), imageIcon.getIconHeight());

        for (Iterator iter = m_widget.getConnectors().iterator(); iter.hasNext();) {
            Connector connector = (Connector) iter.next();
            ConnectorUI connectorUI = new ConnectorUI(connector);
            connector.setConnectorUI(connectorUI);
            m_image.add(connectorUI);
        }

        m_name = new JLabel(m_widget.getName(), JLabel.CENTER);
        m_name.setOpaque(false);

        add(m_image, BorderLayout.CENTER);
        add(m_name, BorderLayout.SOUTH);

        setBounds(m_widget.getX(), m_widget.getY(),
                (int) getPreferredSize().getWidth(), (int) getPreferredSize().getHeight());

        m_popupMenu = createPopupMenu();

        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                m_clickX = e.getX();
                m_clickY = e.getY();

                maybeShowPopup(e);
            }

            public void mouseReleased(MouseEvent e) {
                maybeShowPopup(e);
            }

            private void maybeShowPopup(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    m_popupMenu.show(e.getComponent(),
                            e.getX(), e.getY());
                }
            }
        });

        addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent e) {
                Container container = m_widget.getModel().getModelUI().getContentPane();

                int x = e.getX() + getX() - m_clickX;
                int y = e.getY() + getY() - m_clickY;

                if (x < 0) {
                    x = 0;
                }
                if (x >= container.getWidth() - getWidth()) {
                    x = container.getWidth() - getWidth();
                }

                if (y < 0) {
                    y = 0;
                }
                if (y >= container.getHeight() - getHeight()) {
                    y = container.getHeight() - getHeight();
                }

                m_widget.setCoordinates(x, y);
            }
        });
    }

    public void touch() {
        boolean repaint = false;

        if (getX() != m_widget.getX() || getY() != m_widget.getY()) {
            setLocation(m_widget.getX(), m_widget.getY());
            repaint = true;
        }

        ImageIconFlip icon = (ImageIconFlip) m_image.getIcon();
        if (icon.isFlipped() != m_widget.isFlipped()) {
            icon.flip();
            repaint = true;
        }

        if (!m_widget.getName().equals(m_name.getText())) {
            m_name.setText(m_widget.getName());
            repaint = true;
        }

        m_widget.getModel().getModelUI().repaint();
    }

    private JPopupMenu createPopupMenu() {
        JPopupMenu popupMenu = new JPopupMenu();

        JMenuItem popupFlip = new JMenuItem("Flip");
        popupFlip.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m_widget.flip();
            }
        });
        popupMenu.add(popupFlip);

        JMenuItem popupDelete = new JMenuItem("Delete");
        popupDelete.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m_widget.delete();
            }
        });
        popupMenu.add(popupDelete);

        popupMenu.addSeparator();

        JMenuItem popupStatistics = new JMenuItem("Statistics");
        popupStatistics.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Statistics statistics = m_widget.getStatisticsCopy();
                StatisticsForm form = new StatisticsForm(m_widget.getModel().getModelUI().getFrame(), "Widget Statistics", statistics);

                form.setSize(400, 400);
                form.setVisible(true);

                if (form.isAccepted()) {
                    try {
                        m_widget.setStatistics(statistics);
                    } catch (IllegalArgumentException iae) {
                        JOptionPane.showMessageDialog(WidgetUI.this, iae.getMessage(), "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        });
        popupMenu.add(popupStatistics);

        popupMenu.addSeparator();

        JMenuItem popupProperties = new JMenuItem("Properties");
        popupProperties.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                PropertiesList propertiesList = m_widget.getPropertiesList();
                PropertiesForm form = new PropertiesForm(m_widget.getModel().getModelUI().getFrame(), "Widget Properties", propertiesList);

                form.setSize(400, 400);
                form.setVisible(true);

                if (form.isAccepted()) {
                    try {
                        m_widget.setPropertiesList(propertiesList);
                    } catch (IllegalArgumentException iae) {
                        JOptionPane.showMessageDialog(WidgetUI.this, iae.getMessage(), "Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        });
        popupMenu.add(popupProperties);

        return popupMenu;
    }

    protected abstract String getIconResourceName();
}
