package edu.utexas.cs.surdules.pipes.view;

import edu.utexas.cs.surdules.pipes.model.Widget;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetAllocate;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetDelay;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetRelease;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetResource;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetService;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetSink;
import edu.utexas.cs.surdules.pipes.model.widgets.WidgetSource;
import edu.utexas.cs.surdules.pipes.view.widgets.WidgetAllocateUI;
import edu.utexas.cs.surdules.pipes.view.widgets.WidgetDelayUI;
import edu.utexas.cs.surdules.pipes.view.widgets.WidgetReleaseUI;
import edu.utexas.cs.surdules.pipes.view.widgets.WidgetResourceUI;
import edu.utexas.cs.surdules.pipes.view.widgets.WidgetServiceUI;
import edu.utexas.cs.surdules.pipes.view.widgets.WidgetSinkUI;
import edu.utexas.cs.surdules.pipes.view.widgets.WidgetSourceUI;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public final class WidgetUIFactory {
    private WidgetUIFactory() {
        // nop
    }

    public static WidgetUI createWidgetUIFromWidget(Widget widget) {
        if (widget instanceof WidgetService) {
            return new WidgetServiceUI(widget);
        } else if (widget instanceof WidgetDelay) {
            return new WidgetDelayUI(widget);
        } else if (widget instanceof WidgetAllocate) {
            return new WidgetAllocateUI(widget);
        } else if (widget instanceof WidgetRelease) {
            return new WidgetReleaseUI(widget);
        } else if (widget instanceof WidgetSource) {
            return new WidgetSourceUI(widget);
        } else if (widget instanceof WidgetSink) {
            return new WidgetSinkUI(widget);
        } else if (widget instanceof WidgetResource) {
            return new WidgetResourceUI(widget);
        } else {
            throw new IllegalArgumentException("Invalid Widget type: " + widget.getClass().getName());
        }
    }
}
