package edu.utexas.cs.surdules.pipes.view.forms;

import edu.utexas.cs.surdules.pipes.model.forms.PropertiesItem;
import edu.utexas.cs.surdules.pipes.model.forms.PropertiesList;

import javax.swing.table.AbstractTableModel;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class PropertiesTableModel extends AbstractTableModel {
    /** The names of all columns in this table. */
    private final String[] m_columnNames = {
        "Property",
        "Value"
    };

    /** The configuration for this table. */
    private PropertiesList m_configuration;

    /** The data for this table. */
    private Object[][] m_data;

    public PropertiesTableModel(PropertiesList configuration) {
        m_configuration = configuration;

        m_data = new Object[configuration.getCount()][m_columnNames.length];

        for (int index = 0; index < configuration.getCount(); index++) {
            PropertiesItem item = configuration.getItem(index);
            m_data[index][0] = item.getName();
            m_data[index][1] = item.getValue();
        }
    }

    /** Trivial accessor for the number of columns in the table. */
    public int getColumnCount() {
        return m_columnNames.length;
    }

    /** Trivial accessor for the number of rows in the table. */
    public int getRowCount() {
        return m_data.length;
    }

    /** Trivial accessor for the name of a given column. */
    public String getColumnName(int col) {
        return m_columnNames[col];
    }

    /** Trivial accessor for the data in a particular table cell. */
    public Object getValueAt(int row, int col) {
        return m_data[row][col];
    }

    /**
     * Return true for all cells that are editable by the user.
     */
    public boolean isCellEditable(int row, int col) {
        if (col == 0) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Change table data contents at the given coordinates.
     */
    public void setValueAt(Object value, int row, int col) {
        PropertiesItem item = m_configuration.getItem(row);
        item.setValue(value);
        m_data[row][col] = value;
        fireTableCellUpdated(row, col);
    }
}
