package edu.utexas.cs.surdules.pipes.view.forms;

import edu.utexas.cs.surdules.pipes.model.statistics.Statistics;
import edu.utexas.cs.surdules.pipes.model.statistics.StatletFactory;

import javax.swing.table.AbstractTableModel;
import java.util.Iterator;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class StatisticsTableModel extends AbstractTableModel {
    /** The names of all columns in this table. */
    private final String[] m_columnNames = {
        "Statistic",
        "Enabled"
    };

    /** The configuration for this table. */
    private Statistics m_statistics;

    /** The data for this table. */
    private Object[][] m_data;

    public StatisticsTableModel(Statistics statistics) {
        m_statistics = statistics;

        m_data = new Object[StatletFactory.getStatletCount()][m_columnNames.length];

        int index = 0;
        for (Iterator iter = StatletFactory.getStatletNames().iterator(); iter.hasNext();) {
            String statistic = (String) iter.next();
            m_data[index][0] = statistic;
            m_data[index][1] = new Boolean(statistics.isEnabled(statistic));
            index++;
        }
    }

    /** Trivial accessor for the number of columns in the table. */
    public int getColumnCount() {
        return m_columnNames.length;
    }

    /** Trivial accessor for the number of rows in the table. */
    public int getRowCount() {
        return m_data.length;
    }

    /** Trivial accessor for the name of a given column. */
    public String getColumnName(int col) {
        return m_columnNames[col];
    }

    /** Trivial accessor for the data in a particular table cell. */
    public Object getValueAt(int row, int col) {
        return m_data[row][col];
    }

    /** The class of the object in a particular cell determines the cell editor. */
    public Class getColumnClass(int col) {
        return getValueAt(0, col).getClass();
    }

    /** Return true for all cells that are editable by the user. */
    public boolean isCellEditable(int row, int col) {
        if (col == 0) {
            return false;
        } else {
            return true;
        }
    }

    /** Change table data contents at the given coordinates. */
    public void setValueAt(Object value, int row, int col) {
        String statistic = (String) m_data[row][0];

        // Only boolean values are allowed to be set in this table
        Boolean enable = (Boolean) value;
        if (enable.booleanValue()) {
            m_statistics.enableStatlet(statistic);
        } else {
            m_statistics.disableStatistic(statistic);
        }

        m_data[row][col] = value;

        fireTableCellUpdated(row, col);
    }
}
