package edu.utexas.cs.surdules.pipes.view.simulate;

import edu.utexas.cs.surdules.pipes.model.simulate.Simulator;

import javax.swing.SwingUtilities;
import java.lang.reflect.InvocationTargetException;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class SimulatorThread extends Thread {
    private Simulator m_context;

    private Object m_monitor = new Object();

    private boolean m_stop;
    private boolean m_play;
    private boolean m_step;

    private Throwable m_throwable;

    public SimulatorThread(Simulator context) {
        super("SimulatorThread");

        m_context = context;
    }

    public void run() {
        m_context.start();

        try {
            while (!m_stop) {
                synchronized (m_monitor) {
                    while (!m_play) {
                        try {
                            m_monitor.wait();
                        } catch (InterruptedException ie) {
                            // TODO: better logging
                            ie.printStackTrace();
                        }
                    }
                }

                if (m_context.isFinished()) {
                    break;
                } else {
                    try {
                        SwingUtilities.invokeAndWait(new Runnable() {
                            public void run() {
                                m_context.step();
                            }
                        });
                    } catch (Exception ex) {
                        m_throwable = ex;
                        break;
                    }
                }

                if (m_step) {
                    m_step = false;
                    pause();
                }
            }
        } finally {
            try {
                SwingUtilities.invokeAndWait(new Runnable() {
                    public void run() {
                        m_context.end();
                    }
                });
            } catch (Exception ex) {
                // TODO: better logging
                ex.printStackTrace();
            }
        }
    }

    public void play() {
        m_play = true;
        synchronized (m_monitor) {
            m_monitor.notifyAll();
        }
    }

    public void pause() {
        m_play = false;
    }

    public void step() {
        m_step = true;
        play();
    }

    public void end() {
        m_stop = true;
        play();
    }

    public Throwable getThrowable() {
        if (m_throwable instanceof InvocationTargetException) {
            return ((InvocationTargetException) m_throwable).getTargetException();
        } else {
            return m_throwable;
        }
    }
}
