package edu.utexas.cs.surdules.pipes.view.simulate;

import edu.utexas.cs.surdules.pipes.model.Widget;
import edu.utexas.cs.surdules.pipes.model.simulate.Simulator;
import edu.utexas.cs.surdules.pipes.model.simulate.Event;
import edu.utexas.cs.surdules.pipes.model.statistics.Statistics;
import edu.utexas.cs.surdules.pipes.model.statistics.Statlet;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.filechooser.FileFilter;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;

/**
 * @author <A HREF="mailto:razvan.surdulescu@post.harvard.edu">Razvan Surdulescu</A> (c) 2003
 * <BR>
 * You may use and distribute this software for free provided you include
 * this copyright notice. You may not sell this software, use my name for
 * publicity reasons or modify the code without permission from me.
 */
public class SimulatorUI extends JDialog {
    private Simulator m_context;

    private SimulatorThread m_worker;

    private JCheckBox m_show;
    private JButton m_play, m_pause, m_step, m_stop, m_save;
    private JTree m_log;

    public SimulatorUI(Frame owner, String title, Simulator context) {
        super(owner, title, true);

        m_context = context;

        m_worker = new SimulatorThread(context);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dispose();
            }
        });

        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());

        JPanel controls = new JPanel(new FlowLayout(FlowLayout.RIGHT));

        m_show = new JCheckBox("Show Log");
        m_show.setSelected(true);
        m_play = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/Play24.gif")));
        m_pause = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/Pause24.gif")));
        m_pause.setEnabled(false);
        m_step = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/StepForward24.gif")));
        m_stop = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/Stop24.gif")));
        m_stop.setEnabled(false);
        m_save = new JButton(new ImageIcon(this.getClass().getResource("/media/icons/Save24.gif")));
        m_save.setEnabled(false);

        m_play.setToolTipText("Start the simulation");
        m_play.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m_pause.setEnabled(true);
                m_step.setEnabled(false);
                m_stop.setEnabled(true);

                m_worker.play();
            }
        });

        m_pause.setToolTipText("Pause the simulation");
        m_pause.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m_step.setEnabled(true);
                m_stop.setEnabled(true);

                m_worker.pause();
            }
        });

        m_step.setToolTipText("Step forward in the simulation");
        m_step.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m_stop.setEnabled(true);

                m_worker.step();
            }
        });

        m_stop.setToolTipText("Stop the simulation");
        m_stop.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m_worker.end();
            }
        });

        m_save.setToolTipText("Save the log");
        m_save.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onSave();
            }
        });

        controls.add(m_show);
        controls.add(m_play);
        controls.add(m_pause);
        controls.add(m_step);
        controls.add(m_stop);
        controls.add(m_save);

        m_log = new JTree(new DefaultMutableTreeNode("Simulation"));
        m_log.setShowsRootHandles(true);

        JScrollPane scroll = new JScrollPane(m_log);

        contentPane.add(controls, BorderLayout.SOUTH);
        contentPane.add(scroll, BorderLayout.CENTER);

        setLocationRelativeTo(owner);

        pack();

        m_worker.start();
    }

    private void log(String message) {
        if (!m_show.isSelected()) {
            return;
        }

        DefaultTreeModel treeModel = (DefaultTreeModel) m_log.getModel();

        DefaultMutableTreeNode logNode = null;
        if (treeModel.getChildCount(treeModel.getRoot()) == 0) {
            logNode = new DefaultMutableTreeNode("Log");
            treeModel.insertNodeInto(logNode, (DefaultMutableTreeNode) treeModel.getRoot(), 0);
        } else {
            logNode = (DefaultMutableTreeNode) treeModel.getChild(treeModel.getRoot(), 0);
        }

        DefaultMutableTreeNode entryNode = new DefaultMutableTreeNode(message);
        treeModel.insertNodeInto(entryNode, logNode, treeModel.getChildCount(logNode));

        m_log.scrollPathToVisible(new TreePath(treeModel.getPathToRoot(entryNode)));
    }

    private String stateToString(Event event) {
        switch (event.getState()) {
            case Event.ENTER:
                return "entering into";
            case Event.DEQUEUE:
                return "dequeued into";
            case Event.LEAVE:
                return "leaving from";
            default:
                // this should never happen
                throw new Error("Unrecognized state: " + event.getState());
        }
    }

    public void onEvent(Event event) {
        String message = "Time " + event.getTime() + ": " +
                "Transaction #" + event.getTransactionID() +
                " " + stateToString(event) +
                " " + event.getWidget().getName();

        log(message);
    }

    public void onSuspendEvent(Event event) {
        String message = "Time " + event.getTime() + ": " +
                "Suspended: Transaction #" + event.getTransactionID() +
                " " + stateToString(event) +
                " " + event.getWidget().getName();

        log(message);
    }

    public void onDestroyEvent(Event event) {
        String message = "Time " + event.getTime() + ": " +
                "Finished: Transaction #" + event.getTransactionID() +
                " " + stateToString(event) +
                " " + event.getWidget().getName();

        log(message);
    }

    public void onEnd() {
        m_play.setEnabled(false);
        m_pause.setEnabled(false);
        m_step.setEnabled(false);
        m_stop.setEnabled(false);
        m_save.setEnabled(true);

        if (m_worker.getThrowable() != null) {
            m_worker.getThrowable().printStackTrace();
            JOptionPane.showMessageDialog(this,
                    "An error occured during simulation:\n" +
                    m_worker.getThrowable().getMessage() + "\n" +
                    "See console for complete stack trace",
                    "Simulation Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
        }

        DefaultTreeModel treeModel = (DefaultTreeModel) m_log.getModel();

        DefaultMutableTreeNode statisticsNode = new DefaultMutableTreeNode("Statistics");
        treeModel.insertNodeInto(statisticsNode, (DefaultMutableTreeNode) treeModel.getRoot(),
                treeModel.getChildCount(treeModel.getRoot()));

        for (Iterator widgets = m_context.getModel().getWidgets().iterator(); widgets.hasNext();) {
            Widget widget = (Widget) widgets.next();
            Statistics statistics = widget.getStatistics();
            if (statistics.getStatletCount() > 0) {
                DefaultMutableTreeNode widgetNode = new DefaultMutableTreeNode("Widget '" + widget.getName() + "'");
                treeModel.insertNodeInto(widgetNode, statisticsNode, treeModel.getChildCount(statisticsNode));

                DefaultMutableTreeNode lastStatletNode = null;
                for (Iterator statlets = statistics.getStatlets().values().iterator(); statlets.hasNext();) {
                    Statlet statlet = (Statlet) statlets.next();

                    lastStatletNode = new DefaultMutableTreeNode(statlet.getName() + ":" + statlet.getResult());
                    treeModel.insertNodeInto(lastStatletNode, widgetNode, treeModel.getChildCount(widgetNode));
                }

                m_log.scrollPathToVisible(new TreePath(treeModel.getPathToRoot(lastStatletNode)));
            }
        }
    }

    private void printNode(PrintWriter log, DefaultMutableTreeNode logNode, String indent) {
        log.println(indent + logNode.toString());
        for (int i = 0; i < logNode.getChildCount(); i++) {
            DefaultMutableTreeNode child = (DefaultMutableTreeNode) logNode.getChildAt(i);
            printNode(log, child, indent + "\t");
        }
    }

    private void onSave() {
        JFileChooser fc = new JFileChooser(System.getProperty("user.dir"));

        fc.setFileFilter(new FileFilter() {
            public String getDescription() {
                return "LOG files (*.log)";
            }

            public boolean accept(File f) {
                return f.isDirectory() || f.getName().toLowerCase().endsWith(".log");
            }
        });

        if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
            PrintWriter log = null;
            try {
                File selectedFile = fc.getSelectedFile();
                if (!selectedFile.getName().toLowerCase().endsWith(".log")) {
                    selectedFile = new File(selectedFile.getCanonicalPath() + ".log");
                }

                log = new PrintWriter(new FileOutputStream(selectedFile));
            } catch (IOException ioe) {
                JOptionPane.showMessageDialog(this, ioe.getMessage(), "IO Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
                return;
            }

            DefaultTreeModel treeModel = (DefaultTreeModel) m_log.getModel();

            DefaultMutableTreeNode logNode = (DefaultMutableTreeNode) treeModel.getChild(treeModel.getRoot(), 0);
            printNode(log, logNode, "");

            DefaultMutableTreeNode statisticsNode = (DefaultMutableTreeNode) treeModel.getChild(treeModel.getRoot(), 1);
            printNode(log, statisticsNode, "");

            log.close();

            if (log.checkError()) {
                JOptionPane.showMessageDialog(this, "Error occured while writing to log file", "IO Error", JOptionPane.OK_OPTION | JOptionPane.ERROR_MESSAGE);
            }
        }
    }
}
